/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.communication
 * File:    ConnectionFactory.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.communication;

import de.fhhannover.inform.ifmap.exception.ConnectionCreationException;

public class ConnectionFactory {

	public static Connection createConnection(ConnectionParameter params)
			throws ConnectionCreationException {
		if (params == null) {
			throw new ConnectionCreationException("params == null");
		}
		ConnectionParameter tmp = createConnectionParameter(params.getName(),
				params.getEndpoint(), params.getKeystore(),
				params.getKeystorePass(), params.getTruststore(),
				params.getTruststorePass(), params.isBasicAuthEnabled(),
				params.getBasicauthUser(), params.getBasicauthPass(),
				params.isAutoSubscribe());
		return new Connection(tmp);
	}

	public static ConnectionParameter createConnectionParameter(String name)
			throws ConnectionCreationException {
		ConnectionParameter params = new ConnectionParameter();
		params.setName(name);
		return params;
	}

	public static ConnectionParameter createConnectionParameter(String name,
			String url, String keyPath, String keyPass, String trustPath,
			String trustPass, boolean enableBasicAuth, String basicUser,
			String basicPass, boolean autoSubscribe)
			throws ConnectionCreationException {
		/**
		 * @TODO do checks here (null, path, passwords)
		 */
		ConnectionParameter params = new ConnectionParameter();
		params.setName(name);
		params.setEndpoint(url);
		params.setKeystore(keyPath);
		params.setKeystorePass(keyPass);
		params.setTruststore(trustPath);
		params.setTruststorePass(trustPass);
		params.setBasicAuthEnabled(enableBasicAuth);
		params.setBasicauthUser(basicUser);
		params.setBasicauthPass(basicPass);
		params.setAutoSubscribe(autoSubscribe);
		return params;
	}

}
