/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.view
 * File:    MainFrame.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.fhhannover.inform.ifmap.communication.Connection;
import de.fhhannover.inform.ifmap.view.component.MetadataPanel;
import de.fhhannover.inform.ifmap.view.component.NavigationPanel;
import de.fhhannover.inform.ifmap.view.component.ToolbarPanel;

@SuppressWarnings("serial")
public class MainFrame extends JFrame {
	ViewController mViewController;
	MetadataPanel metadataPanel;
	private JPanel statusPanel;
	NavigationPanel navigationPanel;
	ToolbarPanel toolbarPanel;
	JTabbedPane tabbedPane;
	JLabel status;	

	public MainFrame(ViewController vc) {
		mViewController = vc;
		init();
	}

	private void init() {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			UIManager.put("PopupMenu.consumeEventOnClose", Boolean.TRUE);
		} catch (Throwable e) {
			e.printStackTrace();
		}
		Container mainPanel = this.getContentPane();
		mainPanel.setLayout(new BorderLayout());

		toolbarPanel = new ToolbarPanel();		
		toolbarPanel.setBorder(new EmptyBorder(0, 0, 0, 0));
		getContentPane().add(toolbarPanel, BorderLayout.NORTH);

		JSplitPane splitPane1 = new JSplitPane();
		splitPane1.setDividerSize(2);
		splitPane1.setOrientation(JSplitPane.VERTICAL_SPLIT);
		getContentPane().add(splitPane1, BorderLayout.CENTER);

		JSplitPane splitPane2 = new JSplitPane();
		splitPane2.setDividerSize(2);
		splitPane1.setLeftComponent(splitPane2);

		navigationPanel = new NavigationPanel();
		splitPane2.setLeftComponent(navigationPanel);
		
		tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
		tabbedPane.setFocusable(false);
		splitPane2.setRightComponent(tabbedPane);

		metadataPanel = new MetadataPanel();
		splitPane1.setRightComponent(metadataPanel);
		splitPane1.setDividerLocation(450);

		statusPanel = new JPanel();
		statusPanel
				.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		FlowLayout flowLayout = (FlowLayout) statusPanel.getLayout();
		flowLayout.setAlignment(FlowLayout.RIGHT);
		getContentPane().add(statusPanel, BorderLayout.SOUTH);
		statusPanel.setPreferredSize(new Dimension(getWidth(), 24));

		status = new JLabel(
				"IronGui started successfully.");
		statusPanel.add(status);		
		
		/*********************************************************************************
		 * ActionListener
		 */
		
		// Connection Button
		toolbarPanel.buttonConnections.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				mViewController.showConnectionDialog();
			}
		});
		
		// Quick subscribe
		toolbarPanel.buttonQuickSubscribe.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				mViewController.showQuickSubscribe();
			}
		});
		
		// Tab selected
		tabbedPane.addChangeListener(new ChangeListener() {
			
			@Override
			public void stateChanged(ChangeEvent arg0) {
				mViewController.tabChanged();				
			}
		});
		
		// A single connection in Navigator-Panel is selected
		navigationPanel.panelList.addMouseListener(new MouseAdapter() {
			
			@Override
			public void mouseClicked(MouseEvent e) {
		        if (e.getClickCount() == 2) {
		        	int index = navigationPanel.panelList.locationToIndex(e.getPoint());
		        	DefaultListModel model = (DefaultListModel) navigationPanel.panelList.getModel();
		            Connection con = (Connection) model.get(index);
		        	mViewController.selectConnection(con);
		        }
			}
		});
		
		// Connect
		toolbarPanel.buttonConnect.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				mViewController.toggleConnection();
			}
		});
		// Start dump button selected
		/*toolbarPanel.buttonStartDump.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				mViewController.startDump();
			}
		});*/
		
		// Subscribe...
		toolbarPanel.buttonSubscribe.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				mViewController.showSubsriptionDialog();
			}
		});
		
		// Tree button
		navigationPanel.buttonTree.addItemListener(new ItemListener() {
			
			@Override
			public void itemStateChanged(ItemEvent e) {
				int state = e.getStateChange();
		        if (state == ItemEvent.SELECTED) {
		        	mViewController.treeButtonClicked();
		        }
			}
		});
		
		// Connection button
		navigationPanel.buttonConnections.addItemListener(new ItemListener() {
			
			@Override
			public void itemStateChanged(ItemEvent e) {
				int state = e.getStateChange();
		        if (state == ItemEvent.SELECTED) {
		        	mViewController.conButtonClicked();
		        }
			}
		});
		
		// Publisher button
		
		toolbarPanel.buttonPublisher.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				mViewController.showPublisherDialog();
			}
		});
	}

	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Throwable e) {
			e.printStackTrace();
		}		
		MainFrame f = new MainFrame(null);
		f.setSize(1024, 768);
		f.setLocationRelativeTo(null);
		f.setVisible(true);
	}
}
